import json
import os
from typing import Any, Dict, List

import requests

from ..bridge import BridgeUsername
from ..error import SlackException
from ..github.models.pull_request import PullRequest
from ..github.models.user import User


class Client:
    _usernames: List[BridgeUsername]
    _webhook_url: str

    def __init__(self, usernames: List[BridgeUsername]) -> None:
        self._usernames = usernames
        self._webhook_url = os.environ["SLACK_URL"]

    def _convert_github_to_slack(self, user: User) -> str:
        for username in self._usernames:
            if username.github == user.login:
                return username.slack
        else:
            return user.login

    def _get_section(self, pull: PullRequest) -> List[Dict[str, Any]]:
        # title section
        title_section: Dict[str, Any] = {
            "type": "section",
            "text": {
                "type": "mrkdwn",
                "text": f"[#{pull.number}] <{pull.html_url}|{pull.title}> (_{pull.user.login}_)",
            },
            "accessory": {
                "type": "button",
                "text": {
                    "type": "plain_text",
                    "text": "Review",
                },
                "url": pull.html_url,
            },
        }

        # reviewer section
        reviewer_section: Dict[str, Any] = {"type": "context", "elements": []}
        if pull.requested_reviewers:
            reviewer_section["elements"] += [
                {
                    "type": "plain_text",
                    "text": "Waiting on",
                },
            ]
            for reviewer in pull.requested_reviewers:
                reviewer_section["elements"] += [
                    {
                        "type": "image",
                        "image_url": reviewer.avatar_url,
                        "alt_text": reviewer.login,
                    },
                    {"type": "mrkdwn", "text": self._convert_github_to_slack(user=reviewer)},
                ]
        else:
            reviewer_section["elements"] += [{"type": "plain_text", "text": "Waiting review by anyone."}]

        return [
            title_section,
            reviewer_section,
        ]

    def _build_block(self, repo: str, pulls: List[PullRequest], total_pulls: int) -> List[Dict[str, Any]]:
        blocks = [
            {
                "type": "section",
                "text": {"type": "mrkdwn", "text": f"*Pending review on _<https://github.com/{repo}|{repo}>_*"},
            },
        ]

        for section in map(lambda pull: self._get_section(pull=pull), pulls):
            blocks += section

        if len(pulls) < total_pulls:
            blocks += [
                {
                    "type": "section",
                    "text": {
                        "type": "mrkdwn",
                        "text": f"Number of remaining: *{total_pulls - len(pulls)}*",
                    },
                    "accessory": {
                        "type": "button",
                        "text": {
                            "type": "plain_text",
                            "text": "Watch",
                        },
                        "url": f"https://github.com/{repo}/pulls",
                    },
                },
            ]

        blocks += [
            {
                "type": "divider",
            },
        ]

        return blocks

    def post(self, repo: str, pulls: List[PullRequest], total_pulls: int) -> None:
        if not pulls:
            return

        payload: Dict[str, Any] = {
            "text": f"Waiting your review on {repo}.",
            "blocks": self._build_block(repo=repo, pulls=pulls, total_pulls=total_pulls),
        }

        res = requests.post(url=self._webhook_url, data=json.dumps(payload))
        if res.status_code != 200:
            raise SlackException(status_code=res.status_code, response=res.json())



from typing import Any, Dict


class _AbstractException(Exception):
    status_code: int
    response: Dict[Any, Any]

    def __init__(self, status_code: int, response: Dict[Any, Any]) -> None:
        self.status_code = status_code
        self.response = response


class GitHubException(_AbstractException):
    detail: str

    def __init__(self, status_code: int, response: Dict[Any, Any], detail: str) -> None:
        self.status_code = status_code
        self.response = response
        self.detail = detail
        super().__init__(status_code=status_code, response=response)


class SlackException(_AbstractException):
    pass



import urllib.parse
from typing import Any, Dict, List

import requests

from ..error import GitHubException
from .models.pull_request import PullRequest

GITHUB_API_BASE_URL = "https://api.github.com"


class Client:
    _github_token: str

    def __init__(self, token: str) -> None:
        self._github_token = token

    def get_pulls(self, repo: str, limit: int) -> List[PullRequest]:
        api_url = f"{GITHUB_API_BASE_URL}/repos/{repo}/pulls"
        params: Dict[str, Any] = {"state": "open", "sort": "created", "per_page": 25, "page": 1}
        headers = {
            "Authorization": f"bearer {self._github_token}",
            "Accept": "application/vnd.github.v3+json",
        }
        pulls: List[PullRequest] = []
        while True:
            res = requests.get(url=api_url, params=params, headers=headers)
            res_json = res.json()
            if res.status_code != 200:
                raise GitHubException(status_code=res.status_code, response=res_json, detail="get_pulls")
            if not res_json:
                break

            # filter not draft pull requests
            items = filter(lambda item: not item.draft, map(lambda item: PullRequest(**item), res_json))
            pulls += items

            if len(pulls) >= limit:
                pulls = pulls[:limit]
                break

            value = params.get("page", 1)
            params["page"] = value + 1

        return pulls

    def get_total_pulls(self, repo: str) -> int:
        api_url = (
            f"{GITHUB_API_BASE_URL}/search/issues?q={urllib.parse.quote(f'type:pr repo:{repo} state:open')}&per_page=1"
        )
        headers = {
            "Authorization": f"bearer {self._github_token}",
            "Accept": "application/vnd.github.v3.text-match+json",
        }
        res = requests.get(url=api_url, headers=headers)
        res_json = res.json()
        if res.status_code != 200:
            raise GitHubException(status_code=res.status_code, response=res_json, detail="get_total_pulls")
        total_count = res_json["total_count"]

        return total_count



class User(object):
    login: str
    user_id: int
    user_type: str
    avatar_url: str

    def __init__(self, **kwargs) -> None:
        self.login = kwargs["login"]
        self.user_id = kwargs["id"]
        self.user_type = kwargs["type"]
        self.avatar_url = kwargs["avatar_url"]

    def __str__(self) -> str:
        user = f"@{self.login}"
        if self.user_type != "User":
            user += f" [{self.user_type}]"
        return user



class BridgeUsername:
    github: str
    slack: str

    def __init__(self, **kwargs) -> None:
        self.github = kwargs["github"]
        self.slack = kwargs["slack"]
        if not self.slack.startswith("@"):
            self.slack = f"@{self.slack}"



from typing import List, Optional

from .user import User


class PullRequest(object):
    pr_id: int
    node_id: str
    html_url: str
    number: int
    state: str
    locked: bool
    title: str
    user: User
    assignee: User
    requested_reviewers: List[User]
    draft: Optional[bool]

    def __init__(self, **kwargs) -> None:
        self.pr_id = kwargs["id"]
        self.node_id = kwargs["node_id"]
        self.html_url = kwargs["html_url"]
        self.number = kwargs["number"]
        self.state = kwargs["state"]
        self.locked = kwargs["locked"]
        self.title = kwargs["title"]
        self.user = User(**kwargs["user"])
        self.assignee = kwargs.get("assignee", [])
        self.requested_reviewers = list(map(lambda item: User(**item), kwargs["requested_reviewers"]))
        self.draft = kwargs["draft"]

    def __str__(self) -> str:
        title = f"Pull Request [#{self.number}] {self.title} from {str(self.user)}"
        if self.draft:
            title += "in draft"
        return title



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gs_reminder',
 'gs_reminder.github',
 'gs_reminder.github.models',
 'gs_reminder.slack']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0', 'requests>=2.27.1,<3.0.0']

entry_points = \
{'console_scripts': ['gs-reminder = gs_reminder.notifier:main']}

setup_kwargs = {
    'name': 'gs-reminder',
    'version': '0.3.2',
    'description': 'Notify Slack of a review of Pull Requests in the GitHub repository.',
    'long_description': '# github-pr-slack-reminder\n\n[![Codacy Badge](https://app.codacy.com/project/badge/Grade/380a539992d941f0a6d9c045c48c580c)](https://www.codacy.com/gh/nnsnodnb/github-pr-slack-reminder/dashboard?utm_source=github.com&amp;utm_medium=referral&amp;utm_content=nnsnodnb/github-pr-slack-reminder&amp;utm_campaign=Badge_Grade)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\n[![PyPI Package version](https://badge.fury.io/py/gs-reminder.svg)](https://pypi.org/project/gs-reminder)\n[![Python Supported versions](https://img.shields.io/pypi/pyversions/gs-reminder.svg)](https://pypi.org/project/gs-reminder)\n[![format](https://img.shields.io/pypi/format/gs-reminder.svg)](https://pypi.org/project/gs-reminder)\n[![implementation](https://img.shields.io/pypi/implementation/gs-reminder.svg)](https://pypi.org/project/gs-reminder)\n[![LICENSE](https://img.shields.io/pypi/l/gs-reminder.svg)](https://pypi.org/project/gs-reminder)\n\nNotify Slack of a review of Pull Requests in the GitHub repository.\n\n## Environments\n\n- Python 3.7 or later\n  - poetry\n\n## Usage\n\n```shell\npip install gs-reminder\ngs-reminder -r nnsnodnb/github-pr-slack-reminder -u examples/username.json\n```\n\n### Environment variables\n\n- `GITHUB_TOKEN`\n  - Required\n  - Your GitHub Personal Access Token.\n    - Create https://github.com/settings/tokens\n- `SLACK_URL`\n  - Required\n  - Incoming webhook\'s url of Slack app.\n\n### Options\n\n- `--repo` or `-r`\n  - Required\n  - Your GitHub repository name. (ex. `nnsnodnb/github-pr-slack-reminder`)\n- `--file-username` or `-u`\n  - Optional\n  - Corresponding files for GitHub and Slack usernames. (ex. `examples/username.json`)\n    ```json\n    [\n      {\n        "github": "nnsnodnb",\n        "slack": "yuya.oka"    \n      }\n    ]\n    ```\n\n- `--limit` or `-l`\n  - Optional\n  - Number of Pull Requests to notify Slack. Max: 20 (default: 20)\n\n## Example Result\n\n<img src="https://user-images.githubusercontent.com/9856514/168442310-af165e75-7329-4a37-8e67-3f2635c549ac.png" alt="example result" width="500px">\n\n## License\n\nThis software is licensed under the MIT License.\n',
    'author': 'Yuya Oka',
    'author_email': 'nnsnodnb@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/nnsnodnb/github-pr-slack-reminder',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import json
import os
from pathlib import Path
from typing import List, Optional

import click

from .bridge import BridgeUsername
from .github import api as github_api
from .slack import api as slack_api


def get_bridge_usernames(file_username: Optional[str]) -> List[BridgeUsername]:
    if file_username is None:
        return []
    return list(map(lambda item: BridgeUsername(**item), json.loads(Path(file_username).read_text())))


@click.command(
    help="""GitHub repository's pull requests slack notifier\n
Required environments variables\n
    - GITHUB_TOKEN: Your GitHub Personal Access Token.\n
    - SLACK_URL: Your Slack webhook url."""
)
@click.option(
    "--repo",
    "-r",
    type=str,
    help="GitHub repository name (owner/repo).",
    required=True,
)
@click.option(
    "--file-username",
    "-u",
    type=str,
    help="Corresponding files for GitHub and Slack usernames. (see. example in README.md)",
)
@click.option(
    "--limit",
    "-l",
    type=int,
    default=20,
    help="Number of Pull Requests to notify Slack.",
    required=True,
)
def main(repo: str, file_username: Optional[str], limit: int) -> None:
    if limit > 20:
        raise ValueError("Cannot set more than 20 items.")

    gh = github_api.Client(token=os.environ["GITHUB_TOKEN"])
    # get pull requests
    pulls = gh.get_pulls(repo=repo, limit=limit)
    # get total pull requests count
    total_pulls = gh.get_total_pulls(repo=repo)
    # get GitHub and GitHub bridge username
    usernames = get_bridge_usernames(file_username=file_username)

    # send slack
    sl = slack_api.Client(usernames=usernames)
    sl.post(repo=repo, pulls=pulls, total_pulls=total_pulls)


if __name__ == "__main__":
    main()
